﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.BioBoost.Data {
  [StorableClass]
  [Item("LogisticAction", "Summarizes cost factors and emissions for a logistic action.")]
  public class LogisticAction : DataItem {

    #region Parameters
    public IValueParameter<DoubleValue> InvestmentParameter { get { return (IValueParameter<DoubleValue>)Parameters["Investment"]; } }
    public IValueParameter<DoubleValue> LaborParameter { get { return (IValueParameter<DoubleValue>)Parameters["Labor"]; } }
    public IValueParameter<DoubleValue> FuelParameter { get { return (IValueParameter<DoubleValue>)Parameters["Fuel"]; } }
    public IValueParameter<DoubleValue> MaintenanceParameter { get { return (IValueParameter<DoubleValue>)Parameters["Maintenance"]; } }
    public IValueParameter<DoubleValue> OtherParameter { get { return (IValueParameter<DoubleValue>)Parameters["Other"]; } }
    public IValueParameter<ValueParameterCollection> EmissionsParameter {
      get { return (IValueParameter<ValueParameterCollection>)Parameters["Emissions"]; }
    }
    #endregion

    #region Parameter Values
    public double Investment { get { return InvestmentParameter.Value.Value; } }
    public double Labor { get { return LaborParameter.Value.Value; } }
    public double Fuel { get { return FuelParameter.Value.Value; } }
    public double Maintenance { get { return MaintenanceParameter.Value.Value; } }
    public double Other { get { return OtherParameter.Value.Value; } set { OtherParameter.Value = new DoubleValue(value); } }

    public ValueParameterCollection Emissions { get { return EmissionsParameter.Value; } }
    #endregion

    #region Construction & Cloning

    public LogisticAction() {
      Parameters.Add(new ValueParameter<DoubleValue>("Investment", "The investment cost per unit."));
      Parameters.Add(new ValueParameter<DoubleValue>("Labor", "The labor cost per unit."));
      Parameters.Add(new ValueParameter<DoubleValue>("Fuel", "The fuelt cost per unit."));
      Parameters.Add(new ValueParameter<DoubleValue>("Maintenance", "The maintenance cost per unit."));
      Parameters.Add(new ValueParameter<DoubleValue>("Other", "other costs per unit."));
      Parameters.Add(new ValueParameter<ValueParameterCollection>("Emissions", "The set of emission amounts per unit."));
    }

    [StorableConstructor]
    protected LogisticAction(bool isDeserializing) : base(isDeserializing) { }

    protected LogisticAction(LogisticAction orig, Cloner cloner) : base(orig, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LogisticAction(this, cloner);
    }

    #endregion

    public override bool IsEquivalentTo(DataItem other) {
      return false;
    }
  }
}
